import os
from flask import Flask


def create_app(test_config=None):
    # https://flask.palletsprojects.com/en/1.1.x/tutorial/factory/

    # create and configure the app
    app = Flask(
        __name__, 
        instance_relative_config=True,
        instance_path=os.path.abspath(os.path.join(
            os.path.dirname(os.path.dirname(__file__)), 
            'instance'
        )),
        template_folder=os.path.abspath(os.path.join(
            os.path.dirname(__file__), 
            'templates'
        )),
        static_folder=os.path.abspath(os.path.join(
            os.path.dirname(__file__), 
            'static'
        )),
    )

    # print("Instance Path   = {}".format(app.instance_path))
    # print("Root Path       = {}".format(app.root_path))
    # print("Templates Path  = {}".format(app.template_folder))
    # print("Static Path     = {}".format(app.static_folder))

    if test_config is None:
        # load the instance config, if it exists, when not testing
        app.config.from_pyfile("config.py", silent=True)
    else:
        # load the test config if passed in
        app.config.from_mapping(test_config)

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    # must be done after db initialization
    with app.app_context():
        # import routes
        from .routes import main_route

        # import services
        # ...
        from .services import payment_service

        # register routes
        app.register_blueprint(main_route.bp)
        # register services
        # ...
        app.register_blueprint(payment_service.bp)

        return app
